#ifndef PAGEDEFS_H
#define PAGEDEFS_H
#include <SupportDefs.h>
#include <Rect.h>
#include "PrinterInfo.h"
typedef enum
{
  PAGE_USER_DEFINE = 0,
  PAGE_LEGAL = 1,
  PAGE_LETTER = 2,
  PAGE_EXECUTIVE = 3,
  PAGE_HALFLETTER = 4,
  
  PAGE_A4 = 5,
  PAGE_A5 = 6,
  PAGE_A6 = 7,
  PAGE_B5 = 8,

  PAGE_ENVELOPE_10 = 9,
  PAGE_ENVELOPE_DL = 10,
  PAGE_ENVELOPE_C6 = 11,
  PAGE_ENVELOPE_132X220 = 12,

  PAGE_POSTCARD = 13,
  PAGE_2WAY_POSTCARD = 14,  
  
  PAGE_INDEXCARD_5X8 = 15,
  PAGE_INDEXCARD_8X10 = 16,
  PAGE_PHOTO_4X6 = 17,
  PAGE_PHOTO_100X150 = 18,
  PAGE_PHOTO_200X300 = 19,
  PAGE_PHOTO_PANORAMIC = 20
} page_kind_t;

typedef struct
{
  float width;
  float height;
} st_page_size;
  
// scale in 1/360 dot per inch
typedef struct
{
  uint16 left;
  uint16 width;
  uint16 top;
  uint16 height;   
} st_printable_area;
  
typedef struct 
{
  page_kind_t page_kind;
  char *name_inch;
  char *name_cm;
  st_page_size max_inch_size;
  st_page_size max_cm_size;
  st_printable_area printable_area_nornal;
  st_printable_area printable_area_center;
} st_page_info;

static st_page_info g_support_page_info[] =
{
  // 																										PAGE SIZE (MAX) INCH	PAGE SIZE (MAX)CM	PRINTRECT NORMAL			PRINTRECT CENTER
  {PAGE_USER_DEFINE,		{"User-define"},					{"User-define"},							{0.0f,0.0f},			{0.0f,0.0f},		{0,0,0,0},					{0,0,0,0}},

  {PAGE_LEGAL,				{"Legal(8.5x14in)"}, 				{"Legal(21.6x35.6cm)"},						{8.5f,14.0f},			{21.6f,35.6f},		{42,2976,42,4800},			{42,2976,198,4644}},

  {PAGE_LETTER,				{"Letter(8.5x11in)"},				{"Letter(21.6x27.9cm)"},					{8.5f,11.0f},			{21.6f,27.9f},		{42,2976,42,3720},			{42,2976,198,3564}},

  {PAGE_EXECUTIVE,			{"Executive(7.25x10.5in)"},			{"Executive(18.4x26.7cm)"},					{7.25f,10.5f},			{18.4f,26.7f},		{42,2526,42,3540},			{42,2526,198,3384}},
  {PAGE_HALFLETTER,			{"Half letter(5.5x8.5in)"},			{"Half letter(14x21.6cm)"},					{5.5f,8.5f},			{14.0f,21.6f},		{42,1896,42,2820},			{42,1896,198,2664}},

  {PAGE_A4,					{"A4(8.3x11.7in)"},					{"A4(21x29.7cm)"},							{8.3f,11.7f},			{21.0f,29.7f},		{42,2892,42,3969},			{42,2892,198,3813}},
  {PAGE_A5,					{"A5(5.8x8.3in)"},					{"A5(14.8x21cm)"},							{5.8f,8.3f},			{14.8f,21.0f},		{42,2014,42,2736},			{42,2014,198,2580}},
  {PAGE_A6,					{"A6(4.1x5.8in)"},					{"A6(10.5x14.8cm)"},						{4.1f,5.8f},			{10.5f,14.8f},		{42,1404,42,1858},			{42,1404,198,1702}},
  {PAGE_B5,					{"B5(7.2x10.1in)"},					{"B5(18.2x25.7cm)"},						{7.2f,10.1f},			{18.2f,25.7f},		{42,2496,42,3403},			{42,2496,198,3247}},
  
  {PAGE_ENVELOPE_10		,	{"#10 Envelope(8.5x4.125in)"},		{"#10 Envelope(21.6x10.5cm)"},				{8.5f,4.125f},			{21.6f,10.5f},		{42,2976,42,1245},			{42,2976,198,1089}},
  {PAGE_ENVELOPE_DL		,	{"DL Envelope(8.5x4.3in)"},			{"DL Envelope(16.2x11.4cm)"},				{8.5f,4.3f},			{16.2f,11.4f},		{42,2976,42,1319},			{42,2976,198,1163}},
  {PAGE_ENVELOPE_C6		,	{"C6 Envelope(6.4x4.5in)"},			{"C6 Envelope(10.5x14.8cm)"},				{6.4f,4.5f},			{10.5f,14.8f},		{42,2212,42,1376},			{42,2212,198,1220}},
  {PAGE_ENVELOPE_132X220,	{"Envelope(8.6x5.2in)"},			{"Envelope(22.0x13.2cm)"},					{8.6f,5.2f},			{22.0f,13.2f},		{42,2976,42,1631},			{42,2976,198,1475}},

  {PAGE_POSTCARD		,	{"Postcard(3.9x5.8in)"},			{"Postcard(10x14.8cm)"},					{3.9f,5.8f},			{10.0f,14.8f},		{42,1333,42,1858},			{42,1333,198,1702}},
  {PAGE_2WAY_POSTCARD	,	{"2Way Postcard(7.9x5.8in)"},		{"2Way Postcard(20x14.8cm)"},				{7.9f,5.8f},			{20.0f,14.8f},		{42,2751,42,1858},			{42,2751,198,1702}},

  {PAGE_INDEXCARD_5X8	,	{"Index card 5x8(5x8in)"},			{"Index card 5x8(12.7x20.3cm)"},			{5.0f,8.0f},			{12.7f,20.3f},		{42,1716,42,2640},			{42,1716,198,2484}},
  {PAGE_INDEXCARD_8X10	,	{"Index card 8x10(8x10in)"},		{"Index card 8x10(20.3x25.4cm)"},			{8.0f,10.0f},			{20.3f,25.4f},		{42,2796,42,3360},			{42,2796,198,3204}},

  {PAGE_PHOTO_4X6		,	{"Photo Paper 4x6(4x6in)"},				{"Photo(11.4x17.5cm)"},					{4.6f,6.9f},			{11.4f,17.5f},		{42,1526,42,2246},			{42,1526,42,2246}},
  {PAGE_PHOTO_100X150	,	{"Photo Paper 100x150(3.9x5.9in)"},		{"Photo 100x150(10x15cm)"},				{3.9f,5.9f},			{10.0f,15.0f},		{42,1333,42,1886},			{42,1333,198,1730}},
  {PAGE_PHOTO_200X300	,	{"Photo Paper 200x300(8.5x13.3in)"},		{"Photo 200x300(21.6x33.8cm)"},		{8.5f,13.3f},			{21.6f,33.8f},		{42,2976,42,4395},			{42,2976,42,4395}},
  {PAGE_PHOTO_PANORAMIC	,	{"Panoramic Photo Paper (8.3x23.4in)"},			{"Panoramic(21.0x59.4cm)"},		{8.3,23.4f},			{21.0f,59.4f},		{42,2892,42,8179},			{42,2892,198,8023}}

};

// value in message (PageSetup)															// type
const char MSGSTORE_NAME_PAPER_RECT[]				= "paper_rect";						// BRect
const char MSGSTORE_NAME_RESOLUTION_X[]				= "xres";							// int32
const char MSGSTORE_NAME_RESOLUTION_Y[]				= "yres";							// int32
const int32 MSGSTORE_PAGESETUP_RESOLUTIONX_DEF		= 360;
const int32 MSGSTORE_PAGESETUP_RESOLUTIONY_DEF		= 360;
const char MSGSTORE_NAME_FIRST_PAGE[]				= "first_page";						// int32
const char MSGSTORE_NAME_LAST_PAGE[]				= "last_page";						// int32
const char MSGSTORE_NAME_PAPER_SIZE[]				= "internal_paper_size";
const char MSGSTORE_NAME_ORIENTATION[]   			= "internal_orientation";			// int32
const int32 MSGSTORE_ORIENTATION_PORTRAIT			= 0;				
const int32 MSGSTORE_ORIENTATION_LANDSCAPE  		= 1;
const char MSGSTORE_NAME_SCALE[]					= "internal_scaling";				// float
const float MSGSTORE_SCALING_DEFAULT				= 100.0f;
const char MSGSTORE_NAME_COPIES[]					= "internal_copies";				// int32
const char MSGSTORE_NAME_PRINTABLE_RECT[]			= "printable_rect";					// BRect
const char MSGSTORE_NAME_COLLATE_ORDER[]			= "internal_collate_order";			// bool
const char MSGSTORE_NAME_REVERSE_ORDER[]			= "internal_reverse_order";			// bool
const char MSGSTORE_NAME_ROTATE_180[]				= "internal_rotate_180";			// bool
const char MSGSTORE_NAME_PRINTABLE_STYLE[]			= "internal_printable_style";		// int32
const char MSGSTORE_NAME_USERDEFINE_PAPER_WIDTH[]	= "internal_user_paper_width";		// float
const char MSGSTORE_NAME_USERDEFINE_PAPER_HEIGHT[]	= "internal_user_paper_height";		// float
const char MSGSTORE_NAME_PRINTING_COLOR[]			= "internal_printing_color";		// bool
const char MSGSTORE_NAME_QUALITY_PAPER_KIND[]		= "internal_paper_kind";
const char MSGSTORE_NAME_QUALITY[]					= "internal_quality";
const char MSGSTORE_NAME_QUALITY_RESOLUTION_X[] 	= "internal_quality_xres";
const char MSGSTORE_NAME_QUALITY_RESOLUTION_Y[] 	= "internal_quality_yres";
const char MSGSTORE_NAME_PREVIEW[]					= "internal_preview";

namespace ConvertSpace
{
  const float CM_PER_INCH = 2.541176471f;
  const float ScreenDPI   = 360.0f;
};
float InchToCm(float);
float CmToInch(float);
float InchToScreen(float);
float CmToScreen(float);
float ScreenToInch(uint32);
float ScreenToCm(uint32);

#endif
