#include "PageSetupWindow.h"
#include <Alert.h>
PageSetupWindow::PageSetupWindow(PrinterAddon *addon,uint8 unit)
:BWindow(CenterOfScreen(PageSetupWindowSpace::DEF_WIDTH,PageSetupWindowSpace::DEF_HEIGHT),PageSetupWindowSpace::DEF_TITLE,B_TITLED_WINDOW,B_NOT_RESIZABLE | B_NOT_ZOOMABLE | B_WILL_DRAW)
,faddon(addon),fsem(0),fmsg(NULL),funit(unit)
{
  BuildGUI();
  tb_view->Select(0);
  fsem = create_sem(0,"");
}
void PageSetupWindow::Quit()
{
  delete_sem(fsem); 
  BWindow::Quit();
}
void PageSetupWindow::MessageReceived(BMessage *msg)
{
  switch (msg->what)
  {
    case PageSetupWindowSpace::MSGCODE_COLLATE_ORDER:
    case PageSetupWindowSpace::MSGCODE_REVERSE_ORDER:
    OnOrderChange();
    break;
    
    case PageSetupWindowSpace::MSGCODE_NUM_OF_COPIES:
    OnNumberOfCopiesChange();
    break;
    
    case PageSetupWindowSpace::MSGCODE_PORTRAIT:
    case PageSetupWindowSpace::MSGCODE_LANDSCAPE:
    case PageSetupWindowSpace::MSGCODE_STANDARD:
    case PageSetupWindowSpace::MSGCODE_CENTERED:
    OnStyleChange();
    break;
    
    case PageSetupWindowSpace::MSGCODE_DISCARD:
    OnDiscard();
    break;
    
    case PageSetupWindowSpace::MSGCODE_REVERT:
    OnRevert();
    break;

    case PageSetupWindowSpace::MSGCODE_SAVE:
    OnSave();
    break;
    
    case PageSetupWindowSpace::MSGCODE_PAPERSIZE:
    OnUserDefinePaper();
    break;
    
    case PageSetupWindowSpace::MSGCODE_USERDEFINEPAGEDLG_RESULT:
    OnUserDefinePageDlgResult(msg);
    break;
    
    default:
    BWindow::MessageReceived(msg);
  }
}
void PageSetupWindow::Go(BMessage *msg)
{
  fmsg = msg;
  // set value to what in the message
  Lock();
  SetTo(msg);
  Unlock();
  Show();
  // wait
  acquire_sem(fsem);
  Lock();
  Quit();
}
void PageSetupWindow::BuildGUI()
{
  BRect bound = Bounds();
  bound.bottom++;
  bound.right++;  
  
  // adding mainbox 
  BBox *mainbox = new BBox(bound,NULL);
  mainbox->SetBorder(B_NO_BORDER);
  AddChild(mainbox);
  
  // create tab view
  BRect tb_rect = BRect(5,5,bound.Width() - 5,bound.Height() - 51);
  tb_view = new BTabView(tb_rect,"tabview");
  mainbox->AddChild(tb_view);

  // adjusting tab view RECT  
  BRect tb_sub_rect = tb_view->Bounds();
  tb_sub_rect.bottom -= tb_view->TabHeight() + 2;
  tb_sub_rect.right -= 2;

  // ===================================================  
  // create pape format tab view
  BTab* rtab = new BTab();
  BBox *bg_box = new BBox(tb_sub_rect);
  bg_box->SetBorder(B_NO_BORDER);
  tb_view->AddTab(bg_box,rtab);
  rtab->SetLabel(PageSetupWindowSpace::LB_TAB_PAGEFORMAT);

  // make paper size menu
  int kleft = (int) be_plain_font->StringWidth(PageSetupWindowSpace::LB_PAPERFEED) + (int) be_plain_font->StringWidth(" ");
  mn_paper = new BPopUpMenu(PageSetupWindowSpace::LB_PAPERTYPE_EMPTY); 

  if (funit == PRINTER_UNIT_INCH)
  mn_userdefine_paper_item = new BMenuItem(g_support_page_info[0].name_inch,new BMessage(PageSetupWindowSpace::MSGCODE_PAPERSIZE));
  else
  mn_userdefine_paper_item = new BMenuItem(g_support_page_info[0].name_cm,new BMessage(PageSetupWindowSpace::MSGCODE_PAPERSIZE));

  mn_paper->AddItem(mn_userdefine_paper_item);
  mn_paper->AddSeparatorItem();

  BMenuItem *mn_paper_item;
  for (int i = 1;i < 21;i++)
  {
    if (funit == PRINTER_UNIT_INCH)
    mn_paper_item = new BMenuItem(g_support_page_info[i].name_inch,new BMessage(PageSetupWindowSpace::MSGCODE_PAPERSIZE));
    else
    mn_paper_item = new BMenuItem(g_support_page_info[i].name_cm,new BMessage(PageSetupWindowSpace::MSGCODE_PAPERSIZE));

    mn_paper->AddItem(mn_paper_item);
  }
  BRect r = BRect(8,8,tb_sub_rect.Width() - 8,38);
  BMenuField *mf_paper = new BMenuField(r,NULL,PageSetupWindowSpace::LB_PAPERSIZE,mn_paper);
  mf_paper->SetDivider(kleft);
  mf_paper->SetAlignment(B_ALIGN_LEFT);
  bg_box->AddChild(mf_paper);
  
  r.top += 30;
  r.bottom += 30;
  
  // make paper feed menu
  mn_feed = new BPopUpMenu("paper feed");
  BMenuItem *mn_paper_feed_item = new BMenuItem(PageSetupWindowSpace::LB_CUTSHEET,NULL);
  mn_feed->AddItem(mn_paper_feed_item);
  mn_paper_feed_item->SetMarked(true);
  BMenuField *mf_feed = new BMenuField(r,NULL,PageSetupWindowSpace::LB_PAPERFEED,mn_feed);
  mf_feed->SetDivider(kleft);
  mf_feed->SetAlignment(B_ALIGN_LEFT);
  bg_box->AddChild(mf_feed);
  
  // add printer view box
  BRect r1 = BRect(8,r.bottom + 5,165,tb_sub_rect.Height() - 20);
  BBox *printer_box = new BBox(r1);
  bg_box->AddChild(printer_box);
  BRect bv_style_rect = printer_box->Bounds();
  bv_style_rect.top += 2;
  bv_style_rect.left += 2;
  bv_style_rect.right -= 2;
  bv_style_rect.bottom -= 2;
  bv_style = new PagePrintStyleView(bv_style_rect);
  printer_box->AddChild(bv_style);
  // Adject rect for COPIES
  BRect r2 = BRect(r1.right + 10,r1.top - 10,tb_sub_rect.Width() - 8,r1.top + 75);
  BBox *tempbox;

  BRect tempbox_rect;
  
  // add copies group
  tempbox = new BBox(r2);
  tempbox_rect = tempbox->Bounds();
  tempbox->SetLabel(PageSetupWindowSpace::LB_COPIES_GROUP);
  bg_box->AddChild(tempbox);

  // adding copies bitmap view  
  BRect rect1 = BRect(10,20,tempbox_rect.Width() - 115,70);
  bv_copies = new PageOrderView(rect1);
  tempbox->AddChild(bv_copies);

  // adding textcontrol for # copies
  BRect rect2 = BRect(tempbox_rect.Width() - 104,17,tempbox_rect.Width() - 28,37);  
  tf_copies = new BTextControl(rect2,NULL,PageSetupWindowSpace::LB_COPIES,"1",new BMessage(PageSetupWindowSpace::MSGCODE_NUM_OF_COPIES));
  tf_copies->SetDivider(be_plain_font->StringWidth(PageSetupWindowSpace::LB_COPIES));
  tf_copies->TextView()->SetMaxBytes(2);
  for (uint32 c = 0;c < 255;c++)
  {
    if (c >= '0' && c <= '9')
    tf_copies->TextView()->AllowChar(c);
    else
    tf_copies->TextView()->DisallowChar(c);
  }  
  tempbox->AddChild(tf_copies);

  rect2.right += 20;  
  rect2.top += 23;
  rect2.bottom += 23;

  // adding collate copies option
  chk_collate = new BCheckBox(rect2,NULL,PageSetupWindowSpace::LB_COLLATE,new BMessage(PageSetupWindowSpace::MSGCODE_COLLATE_ORDER));
  tempbox->AddChild(chk_collate);

  rect2.top += 22;
  rect2.bottom += 22;
  
  // adding reverse order option
  chk_reverse = new BCheckBox(rect2,NULL,PageSetupWindowSpace::LB_REVERSE,new BMessage(PageSetupWindowSpace::MSGCODE_REVERSE_ORDER));
  tempbox->AddChild(chk_reverse);  
  
  // adding orientation group
  r2.top = r2.bottom + 4;
  r2.bottom = r2.top + 60;
  
  tempbox = new BBox(r2);
  tempbox->SetLabel(PageSetupWindowSpace::LB_ORIENTATION_GROUP);
  bg_box->AddChild(tempbox);

  // add portrait option
  BRect r3 = BRect(13,15,90,28); 
  rb_portrait = new BRadioButton(r3,NULL,PageSetupWindowSpace::LB_PORTRAIT,new BMessage(PageSetupWindowSpace::MSGCODE_PORTRAIT));
  tempbox->AddChild(rb_portrait);

  // add landscape option
  BRect r4 = BRect(tempbox_rect.Width() - 104,r3.top,tempbox_rect.Width() - 8,r3.bottom);
  rb_landscape = new BRadioButton(r4,NULL,PageSetupWindowSpace::LB_LANDSCAPE,new BMessage(PageSetupWindowSpace::MSGCODE_LANDSCAPE));
  tempbox->AddChild(rb_landscape);

  // add rotate 180 degree option
  BRect r5 = r3;
  
  r5.top += 20;
  r5.bottom += 20;
  r5.right = tempbox_rect.Width() - 10;
  chk_180rotate = new BCheckBox(r5,NULL,PageSetupWindowSpace::LB_180ROTATE,NULL);
  tempbox->AddChild(chk_180rotate);

  // add printable group
  r2.top = r2.bottom + 4;
  r2.bottom = r2.top + 66;
  tempbox = new BBox(r2);
  tempbox->SetLabel(PageSetupWindowSpace::LB_PRINTABLEAREA_GROUP);
  bg_box->AddChild(tempbox);
  
  r3.top += 5;
  r3.bottom += 5;
  rb_parea_standard = new BRadioButton(r3,NULL,PageSetupWindowSpace::LB_STANDARD,new BMessage(PageSetupWindowSpace::MSGCODE_STANDARD));
  tempbox->AddChild(rb_parea_standard);

  r4.top += 5;
  r4.bottom += 5;
  rb_parea_centered = new BRadioButton(r4,NULL,PageSetupWindowSpace::LB_CENTERED,new BMessage(PageSetupWindowSpace::MSGCODE_CENTERED));
  tempbox->AddChild(rb_parea_centered);
  
  // ===================================================  
  // create position tab view
  rtab = new BTab();
  bg_box = new BBox(tb_sub_rect,NULL);
  bg_box->SetBorder(B_NO_BORDER);
  tb_view->AddTab(bg_box,rtab);
  rtab->SetLabel(PageSetupWindowSpace::LB_TAB_POSITION);

  // ===================================================  

  // Adding buttons 
  bound = mainbox->Bounds();
  BRect btn_rect = BRect(10,bound.Height() - 38,100,bound.Height() - 20);
  btn_discard = new BButton(btn_rect,NULL,PageSetupWindowSpace::LB_DISCARD,new BMessage(PageSetupWindowSpace::MSGCODE_DISCARD));
  mainbox->AddChild(btn_discard);

  btn_rect.left = bound.Width() - 200;
  btn_rect.right = btn_rect.left + 90;

  btn_revert = new BButton(btn_rect,NULL,PageSetupWindowSpace::LB_REVERT,new BMessage(PageSetupWindowSpace::MSGCODE_REVERT));
  btn_revert->SetEnabled(false);
  mainbox->AddChild(btn_revert);

  btn_rect.left = bound.Width() - 100;
  btn_rect.right = bound.Width() - 10;
  btn_save = new BButton(btn_rect,NULL,PageSetupWindowSpace::LB_SAVE,new BMessage(PageSetupWindowSpace::MSGCODE_SAVE));
  btn_save->MakeDefault(true);
  mainbox->AddChild(btn_save);
}

void PageSetupWindow::OnOrderChange()
{
  Lock();
  bv_copies->SetTo(chk_collate->Value(),chk_reverse->Value());
  Unlock();
}

void PageSetupWindow::OnStyleChange()
{
  Lock();
  bv_style->SetTo(rb_portrait->Value(),rb_parea_standard->Value());
  Unlock();
}

void PageSetupWindow::OnDiscard()
{
  Lock();
  if (fmsg != NULL)
  fmsg->what = ('baad');
  release_sem(fsem);
  Unlock();   
}
void PageSetupWindow::OnRevert()
{
  Lock();
  Unlock();   
}
void PageSetupWindow::OnSave()
{
  Lock();
  if (fmsg != NULL)
  {
    fmsg->what = (uint32) 'okok';  
    Retrieve(fmsg);
  }
  release_sem(fsem);
  Unlock();   
}
void PageSetupWindow::Retrieve(BMessage *msg)
{
  if (msg != NULL)
  {
    BRect rect;
    int32 int32_value;
    float float_value;
    bool bool_value;   
    // paper_size
    if (msg->FindInt32(MSGSTORE_NAME_PAPER_SIZE,&int32_value) == B_OK)
    msg->ReplaceInt32(MSGSTORE_NAME_PAPER_SIZE,GetPaperSize());
    else
    msg->AddInt32(MSGSTORE_NAME_PAPER_SIZE,GetPaperSize());
    // -------------------------------------------------------
    
    // User Define Paper Width
    if (msg->FindFloat(MSGSTORE_NAME_USERDEFINE_PAPER_WIDTH,&float_value) == B_OK)
    msg->ReplaceFloat(MSGSTORE_NAME_USERDEFINE_PAPER_WIDTH,GetUserDefinePaperWidth());
    else
    msg->AddFloat(MSGSTORE_NAME_USERDEFINE_PAPER_WIDTH,GetUserDefinePaperWidth());

    // User Define Paper Height
    if (msg->FindFloat(MSGSTORE_NAME_USERDEFINE_PAPER_HEIGHT,&float_value) == B_OK)
    msg->ReplaceFloat(MSGSTORE_NAME_USERDEFINE_PAPER_HEIGHT,GetUserDefinePaperHeight());
    else
    msg->AddFloat(MSGSTORE_NAME_USERDEFINE_PAPER_HEIGHT,GetUserDefinePaperHeight());
 
    // printable_style
    if (msg->FindBool(MSGSTORE_NAME_PRINTABLE_STYLE,&bool_value) == B_OK)
    msg->ReplaceBool(MSGSTORE_NAME_PRINTABLE_STYLE,rb_parea_standard->Value() == 1);
    else
    msg->AddBool(MSGSTORE_NAME_PRINTABLE_STYLE,rb_parea_standard->Value() == 1);
 
    // paper_rect 
    if (msg->FindRect(MSGSTORE_NAME_PAPER_RECT,&rect) == B_OK)
    msg->ReplaceRect(MSGSTORE_NAME_PAPER_RECT,GetPaperRect());
    else
    msg->AddRect(MSGSTORE_NAME_PAPER_RECT,GetPaperRect());
    // -------------------------------------------------------

    // printable_rect
    if (msg->FindRect(MSGSTORE_NAME_PRINTABLE_RECT,&rect) == B_OK)
    msg->ReplaceRect(MSGSTORE_NAME_PRINTABLE_RECT,GetPrintableRect());
    else
    msg->AddRect(MSGSTORE_NAME_PRINTABLE_RECT,GetPrintableRect());

    // set printing resolution to 360x360 dpi
    if (msg->FindInt32(MSGSTORE_NAME_RESOLUTION_X,&int32_value) == B_OK)
    msg->ReplaceInt32(MSGSTORE_NAME_RESOLUTION_X,MSGSTORE_PAGESETUP_RESOLUTIONX_DEF);
    else
    msg->AddInt32(MSGSTORE_NAME_RESOLUTION_X,MSGSTORE_PAGESETUP_RESOLUTIONX_DEF);
    if (msg->FindInt32(MSGSTORE_NAME_RESOLUTION_Y,&int32_value) == B_OK)
    msg->ReplaceInt32(MSGSTORE_NAME_RESOLUTION_Y,MSGSTORE_PAGESETUP_RESOLUTIONY_DEF);
    else
    msg->AddInt32(MSGSTORE_NAME_RESOLUTION_Y,MSGSTORE_PAGESETUP_RESOLUTIONY_DEF);
    
    // orientation
    if (msg->FindInt32(MSGSTORE_NAME_ORIENTATION,&int32_value) == B_OK)
    {
      if (rb_portrait->Value())
      msg->ReplaceInt32(MSGSTORE_NAME_ORIENTATION,MSGSTORE_ORIENTATION_PORTRAIT);
      else
      msg->ReplaceInt32(MSGSTORE_NAME_ORIENTATION,MSGSTORE_ORIENTATION_LANDSCAPE);
    }
    else
    {
      if (rb_portrait->Value())
      msg->AddInt32(MSGSTORE_NAME_ORIENTATION,MSGSTORE_ORIENTATION_PORTRAIT);
      else
      msg->AddInt32(MSGSTORE_NAME_ORIENTATION,MSGSTORE_ORIENTATION_LANDSCAPE);
    }
    // -------------------------------------------------------

    // # copies
    if (msg->FindInt32(MSGSTORE_NAME_COPIES,&int32_value) == B_OK)
    msg->ReplaceInt32(MSGSTORE_NAME_COPIES,GetNumberOfCopies());
    else
    msg->AddInt32(MSGSTORE_NAME_COPIES,GetNumberOfCopies());
    // -------------------------------------------------------
    
    // Collate Order    
    if (msg->FindBool(MSGSTORE_NAME_COLLATE_ORDER,&bool_value) == B_OK)
    msg->ReplaceBool(MSGSTORE_NAME_COLLATE_ORDER,chk_collate->Value());
    else
    msg->AddBool(MSGSTORE_NAME_COLLATE_ORDER,chk_collate->Value());
    // -------------------------------------------------------

    // Reverse Order
    if (msg->FindBool(MSGSTORE_NAME_REVERSE_ORDER,&bool_value) == B_OK)
    msg->ReplaceBool(MSGSTORE_NAME_REVERSE_ORDER,chk_reverse->Value());
    else
    msg->AddBool(MSGSTORE_NAME_REVERSE_ORDER,chk_reverse->Value());
    // -------------------------------------------------------
    
    // Rotate
    if (msg->FindBool(MSGSTORE_NAME_ROTATE_180,&bool_value) == B_OK)
    msg->ReplaceBool(MSGSTORE_NAME_ROTATE_180,chk_180rotate->Value());
    else
    msg->AddBool(MSGSTORE_NAME_ROTATE_180,chk_180rotate->Value());
    // -------------------------------------------------------
 
    // scaling
    if (msg->FindFloat(MSGSTORE_NAME_SCALE,&float_value) == B_OK)
    msg->ReplaceFloat(MSGSTORE_NAME_SCALE,MSGSTORE_SCALING_DEFAULT);
    else
    msg->AddFloat(MSGSTORE_NAME_SCALE,MSGSTORE_SCALING_DEFAULT);
    // -------------------------------------------------------
  }
}
int32 PageSetupWindow::GetPaperSize()			// return paper kind
{
  return fpaper_size;
}

float PageSetupWindow::GetUserDefinePaperWidth()	// return user define paper width
{
  return fuserdefine_width;
}
float PageSetupWindow::GetUserDefinePaperHeight()	// return user define paper height
{
  return fuserdefine_height;
}
BRect PageSetupWindow::GetPaperRect()				// return paper rect in pixel
{
  BRect rect;
  rect.left = 0;
  rect.top = 0;
  if (fpaper_size == PAGE_USER_DEFINE)
  {
    if (funit == PRINTER_UNIT_INCH)
    {
      rect.right  = InchToScreen(fuserdefine_width);
      rect.bottom = InchToScreen(fuserdefine_height);
    }
    else
    {
      rect.right  = CmToScreen(fuserdefine_width);
      rect.bottom = CmToScreen(fuserdefine_height);
    } 
  }
  else
  {
    rect.right  = InchToScreen(g_support_page_info[fpaper_size].max_inch_size.width) ;
    rect.bottom = InchToScreen(g_support_page_info[fpaper_size].max_inch_size.height);
  }
  if (rb_portrait->Value())
  return rect;
  return BRect(rect.top,rect.left,rect.bottom,rect.right);
}
BRect PageSetupWindow::GetPrintableRect()			// return printable rect uh.. in pixel
{
  BRect rect;
  if (fpaper_size == PAGE_USER_DEFINE)
  {
    int32 width,height;
    if (funit == PRINTER_UNIT_INCH)
    {
      width  = (int32) (fuserdefine_width * 360.0f);
      height = (int32) (fuserdefine_height * 360.0f);
    } 
    else
    {
      width  = (int32) (CmToInch(fuserdefine_width) * 360.0f);
      height = (int32) (CmToInch(fuserdefine_height) * 360.0f);
    }

    rect.left   = InchToScreen(42.0f / 360.0f);
    rect.right  = InchToScreen((width - 42.0f) / 360.0f);
        
    rect.top    = InchToScreen(42.0f / 360.0f);
    rect.bottom = InchToScreen((height - 354) / 360.0f);
  }
  else
  {
    if (rb_parea_standard->Value())
    {
      rect.left   = InchToScreen(g_support_page_info[fpaper_size].printable_area_nornal.left / 360.0f);
      rect.right  = InchToScreen((g_support_page_info[fpaper_size].printable_area_nornal.left +  g_support_page_info[fpaper_size].printable_area_nornal.width) / 360.0f);
      rect.top    = InchToScreen(g_support_page_info[fpaper_size].printable_area_nornal.top / 360.0f);
      rect.bottom = InchToScreen((g_support_page_info[fpaper_size].printable_area_nornal.top + g_support_page_info[fpaper_size].printable_area_nornal.height) / 360.0f);
    }
    else
    {
      rect.left   = InchToScreen(g_support_page_info[fpaper_size].printable_area_center.left / 360);
      rect.right  = InchToScreen((g_support_page_info[fpaper_size].printable_area_center.left +  g_support_page_info[fpaper_size].printable_area_nornal.width) / 360);
      rect.top    = InchToScreen(g_support_page_info[fpaper_size].printable_area_center.top / 360);
      rect.bottom = InchToScreen((g_support_page_info[fpaper_size].printable_area_center.top + g_support_page_info[fpaper_size].printable_area_nornal.height) / 360);
    }
  }
  if (rb_portrait->Value())
  return rect;
  return BRect(rect.top,rect.left,rect.bottom,rect.right);
}
int32 PageSetupWindow::GetNumberOfCopies()			// return number of copies
{
  int copies = 0;
  char temp[3];
  sprintf(temp,"%s",tf_copies->Text());
  if ((sscanf(temp,"%d",&copies) < 1) || (copies < 0))
  {
    copies = 1;
    tf_copies->SetText("1");
  }
  return copies;
}
void PageSetupWindow::UpdateUserDefinePageSizeMenuLabel()
{
  char temp[256];
  if (funit == PRINTER_UNIT_INCH)
  sprintf(temp,"%s %0.2fx%0.2f inch",g_support_page_info[PAGE_USER_DEFINE].name_inch,fuserdefine_width,fuserdefine_height);
  else
  sprintf(temp,"%s %0.2fx%0.2f cm",g_support_page_info[PAGE_USER_DEFINE].name_cm,fuserdefine_width,fuserdefine_height);
  mn_userdefine_paper_item->SetLabel(temp);  
}
void PageSetupWindow::ReselectPageSizeMenu()
{
  int32 index = fpaper_size;
  if (fpaper_size > PAGE_USER_DEFINE)
  index++;
  mn_paper->ItemAt(index)->SetMarked(true);
}

void PageSetupWindow::OnNumberOfCopiesChange()
{
  int copies;
  char temp[3];

  Lock();
  sprintf(temp,"%s",tf_copies->Text());
  if (sscanf(temp,"%d",&copies) < 1)
  tf_copies->SetText("1");
  else if (copies < 1)
  tf_copies->SetText("1");
  Unlock();
}
void PageSetupWindow::OnUserDefinePaper()
{
  uint32 select_index;
  Lock();
  // get select index
  BMenuItem *markeditem = mn_paper->FindMarked();
  if (markeditem != NULL)
  {
    int32 markedindex = mn_paper->IndexOf(markeditem);
    if (markedindex < 0)
    {
      fpaper_size = select_index = PAGE_A4;
      mn_paper->ItemAt(select_index + 1)->SetMarked(true);
      Unlock();
      return;
    }
    else if (markedindex > 0)
    select_index = markedindex - 1;
    else
    select_index = markedindex;
  }
  else
  {
    fpaper_size = select_index = PAGE_A4;
    mn_paper->ItemAt(select_index + 1)->SetMarked(true);
    Unlock();
    return;
  }

  if (select_index == PAGE_USER_DEFINE)
  {
    if (funit == PRINTER_UNIT_INCH)
    userdlg = new UserDefinePageDlg(funit,
    								ScreenToInch(faddon->MinPageWidth()),
    								ScreenToInch(faddon->MinPageHeight()),
    								ScreenToInch(faddon->MaxPageWidth()),
    								ScreenToInch(faddon->MaxPageHeight()),
  								  	new BMessage(PageSetupWindowSpace::MSGCODE_USERDEFINEPAGEDLG_RESULT),this);
    else
    userdlg = new UserDefinePageDlg(funit,
    								ScreenToCm(faddon->MinPageWidth()),
    								ScreenToCm(faddon->MinPageHeight()),
    								ScreenToCm(faddon->MaxPageWidth()),
    								ScreenToCm(faddon->MaxPageHeight()),
  								  	new BMessage(PageSetupWindowSpace::MSGCODE_USERDEFINEPAGEDLG_RESULT),this);
    userdlg->SetTo(fuserdefine_width,fuserdefine_height);
    userdlg->Show();
  }
  else
  {
    fpaper_size = select_index;
  }  
  Unlock();
}
void PageSetupWindow::OnUserDefinePageDlgResult(BMessage *msg)
{
  int32 result;
  Lock();
  // get current select item
  msg->FindInt32("result",&result);
  if (result == B_OK)
  {
    msg->FindFloat("width",&fuserdefine_width);
    msg->FindFloat("height",&fuserdefine_height);
    // update menu item label
    UpdateUserDefinePageSizeMenuLabel();
    // save last page selection to user define
    fpaper_size = PAGE_USER_DEFINE;
  }
  // re-select the last selection
  ReselectPageSizeMenu();
  Unlock();
}
void PageSetupWindow::SetTo(BMessage *msg)
{
  BString str;
  if (msg == NULL)
  return;
  // Paper Type
  if (msg->FindInt32(MSGSTORE_NAME_PAPER_SIZE,&fpaper_size) != B_OK)
  fpaper_size = PAGE_A4;
  // User Define Width
  if (msg->FindFloat(MSGSTORE_NAME_USERDEFINE_PAPER_WIDTH,&fuserdefine_width) == B_OK)
  {
    if (funit == PRINTER_UNIT_INCH)
    {
      if (fuserdefine_width > ScreenToInch(faddon->MaxPageWidth()))
      fuserdefine_width = ScreenToInch(faddon->MaxPageWidth());
      else if (fuserdefine_width < ScreenToInch(faddon->MinPageWidth()))
      fuserdefine_width = ScreenToInch(faddon->MinPageWidth());
    }
    else
    {
      if (fuserdefine_width > ScreenToCm(faddon->MaxPageWidth()))
      fuserdefine_width = ScreenToCm(faddon->MinPageWidth());
      else if (fuserdefine_width < ScreenToCm(faddon->MinPageWidth()))
      fuserdefine_width = ScreenToCm(faddon->MinPageWidth());
    }
  }
  else
  {
    if (funit == PRINTER_UNIT_INCH)
    fuserdefine_width = ScreenToInch(faddon->MinPageWidth());
    else
    fuserdefine_width = ScreenToCm(faddon->MinPageWidth());
  }
  // User Define Height
  if (msg->FindFloat(MSGSTORE_NAME_USERDEFINE_PAPER_HEIGHT,&fuserdefine_height) == B_OK)
  {
    if (funit == PRINTER_UNIT_INCH)
    {
      if (fuserdefine_height > ScreenToInch(faddon->MaxPageHeight()))
      fuserdefine_height = ScreenToInch(faddon->MaxPageHeight());
      else if (fuserdefine_height < ScreenToInch(faddon->MinPageHeight()))
      fuserdefine_height = ScreenToInch(faddon->MinPageHeight());
    }
    else
    {
      if (fuserdefine_height > ScreenToCm(faddon->MaxPageHeight()))
      fuserdefine_height = ScreenToCm(faddon->MaxPageHeight());
      else if (fuserdefine_height < ScreenToCm(faddon->MinPageHeight()))
      fuserdefine_height = ScreenToCm(faddon->MinPageHeight());
    }
  }
  else
  {
    if (funit == PRINTER_UNIT_INCH)
    fuserdefine_height = ScreenToInch(faddon->MinPageHeight());
    else
    fuserdefine_height = ScreenToCm(faddon->MinPageHeight());
  }  
  UpdateUserDefinePageSizeMenuLabel();
  ReselectPageSizeMenu();
  // -------------------------------------------------------

  // # copies
  int32 p_copies;
  if (msg->FindInt32(MSGSTORE_NAME_COPIES,&p_copies) == B_OK)
  {
    if (p_copies < 0)
    p_copies = 1;
    else if (p_copies > 99)
    p_copies = 99;
  }
  else
  p_copies = 1;

  BString str_copies;
  str_copies << (int32) p_copies;
  tf_copies->SetText(str_copies.String());
  // -------------------------------------------------------
    
  // Collate Order
  bool p_collate_order;
  if (msg->FindBool(MSGSTORE_NAME_COLLATE_ORDER,&p_collate_order) != B_OK)
  {
    p_collate_order = false;
    msg->AddBool(MSGSTORE_NAME_COLLATE_ORDER,p_collate_order);
  }
  chk_collate->SetValue(p_collate_order);
  // -------------------------------------------------------

  // Reverse Order
  bool p_reverse_order;
  if (msg->FindBool(MSGSTORE_NAME_REVERSE_ORDER,&p_reverse_order) != B_OK)
  {
    p_reverse_order = false;
    msg->AddBool(MSGSTORE_NAME_REVERSE_ORDER,p_reverse_order);
  }
  chk_reverse->SetValue(p_reverse_order);
  // -------------------------------------------------------
    
  // Rotate
  bool p_rotate_order;
  if (msg->FindBool(MSGSTORE_NAME_ROTATE_180,&p_rotate_order) != B_OK)
  {
    p_rotate_order = false;
    msg->AddBool(MSGSTORE_NAME_ROTATE_180,p_rotate_order);
  }
  chk_180rotate->SetValue(p_rotate_order);
  // -------------------------------------------------------

  bv_copies->SetTo(p_collate_order,p_reverse_order);

  // orientation
  int32 p_orientation;
  if (msg->FindInt32(MSGSTORE_NAME_ORIENTATION,&p_orientation) == B_OK)
  {
    if ((p_orientation != MSGSTORE_ORIENTATION_PORTRAIT) && (p_orientation != MSGSTORE_ORIENTATION_LANDSCAPE))
    p_orientation = MSGSTORE_ORIENTATION_PORTRAIT;
  }
  else
  {
    p_orientation = MSGSTORE_ORIENTATION_PORTRAIT;
    msg->AddInt32(MSGSTORE_NAME_ORIENTATION,p_orientation);
  }
  
  if (p_orientation == MSGSTORE_ORIENTATION_PORTRAIT)
  {
    rb_landscape->SetValue(false);
    rb_portrait->SetValue(true);
  }
  else
  {
    rb_portrait->SetValue(false);
    rb_landscape->SetValue(true);
  }
  // -------------------------------------------------------
  
  // printable area style
  bool normal_printable_area;
  if (msg->FindBool(MSGSTORE_NAME_PRINTABLE_STYLE,&normal_printable_area) != B_OK)
  normal_printable_area = true;
  
  if (normal_printable_area)
  {
    rb_parea_centered->SetValue(false);
    rb_parea_standard->SetValue(true);
  }
  else
  {
    rb_parea_standard->SetValue(false);
    rb_parea_centered->SetValue(true);
  }
  
  bv_style->SetTo(p_orientation == MSGSTORE_ORIENTATION_PORTRAIT,normal_printable_area);
  // scaling not yet implemented
}
