#include "UserDefinePageDlg.h"
UserDefinePageDlg::UserDefinePageDlg(uint8 unit,float ppage_min_width,float ppage_min_height,float ppage_max_width,float ppage_max_height,BMessage *msg,BLooper *target)
:BWindow(CenterOfScreen(UserDefinePageDlgSpace::DEF_WIDTH,UserDefinePageDlgSpace::DEF_HEIGHT),
		 NULL,
		 B_MODAL_WINDOW_LOOK,
		 B_MODAL_APP_WINDOW_FEEL,
		 B_NOT_RESIZABLE | B_NOT_ZOOMABLE)
,page_min_width(ppage_min_width),page_min_height(ppage_min_height)
,page_max_width(ppage_max_width),page_max_height(ppage_max_height)
,fmsg(msg),ftarget(target),funit(unit)
{
  BuildGUI();
}
UserDefinePageDlg::~UserDefinePageDlg()
{

}
void UserDefinePageDlg::MessageReceived(BMessage *msg)
{
  switch (msg->what)
  {
    case UserDefinePageDlgSpace::MSGCODE_OK:
    OnOK();
    break;
    
    case UserDefinePageDlgSpace::MSGCODE_CANCEL:
    OnCancel();
    break;
    
    case UserDefinePageDlgSpace::MSGCODE_PAGE_WIDTH:
    OnPageWidthChange();
    break;
    
    case UserDefinePageDlgSpace::MSGCODE_PAGE_HEIGHT:
    OnPageHeightChange();
    break;
    
    default:
    BWindow::MessageReceived(msg);
  }
}
void UserDefinePageDlg::BuildGUI()
{
  BRect bound = this->Bounds();
  bound.bottom++;
  bound.right++;
  BBox *mainbox = new BBox(bound,NULL);
  mainbox->SetBorder(B_NO_BORDER);
  AddChild(mainbox);
  
  bound = mainbox->Bounds();
  
  BRect r1 = BRect(50,20,125,35);
  BRect r2 = BRect(130,21,170,36);
  
  // Create Text Control for PageWidth
  tf_width = new BTextControl(r1,NULL,UserDefinePageDlgSpace::LB_PAGE_WIDTH,"0.0",new BMessage(UserDefinePageDlgSpace::MSGCODE_PAGE_WIDTH));
  tf_width->SetDivider(be_plain_font->StringWidth(UserDefinePageDlgSpace::LB_PAGE_HEIGHT));
  for (uint32 c = 0;c < 255;c++)
  {
    if ((c >= '0' && c <= '9') || (c == '.'))
    tf_width->TextView()->AllowChar(c);
    else
    tf_width->TextView()->DisallowChar(c);
  }
  mainbox->AddChild(tf_width);

  if (funit == PRINTER_UNIT_INCH)
  mainbox->AddChild(new BStringView(r2,NULL,"inch"));
  else
  mainbox->AddChild(new BStringView(r2,NULL,"cm"));
  
  r1.top += 30;
  r1.bottom += 30;
  r2.top += 30;
  r2.bottom += 30;
  
  // Create Text Control for PageHeight
  tf_height = new BTextControl(r1,NULL,UserDefinePageDlgSpace::LB_PAGE_HEIGHT,"0.0",new BMessage(UserDefinePageDlgSpace::MSGCODE_PAGE_HEIGHT));
  tf_height->SetDivider(be_plain_font->StringWidth(UserDefinePageDlgSpace::LB_PAGE_HEIGHT));
  for (uint32 c = 0;c < 255;c++)
  {
    if ((c >= '0' && c <= '9') || (c == '.'))
    tf_height->TextView()->AllowChar(c);
    else
    tf_height->TextView()->DisallowChar(c);
  }
  mainbox->AddChild(tf_height);

  if (funit == PRINTER_UNIT_INCH)
  mainbox->AddChild(new BStringView(r2,NULL,"inch"));
  else
  mainbox->AddChild(new BStringView(r2,NULL,"cm"));
  
  BRect rect = BRect(bound.Width() - (160 - 3),bound.Height() - 30,bound.Width() - (90 - 3),bound.Height() - 10);
  btn_cancel = new BButton(rect,NULL,UserDefinePageDlgSpace::LB_CANCEL,new BMessage(UserDefinePageDlgSpace::MSGCODE_CANCEL));
  mainbox->AddChild(btn_cancel);
  rect.left = rect.right + 5;
  rect.right = bound.Width() - 7;
  btn_ok = new BButton(rect,NULL,UserDefinePageDlgSpace::LB_OK,new BMessage(UserDefinePageDlgSpace::MSGCODE_OK));
  mainbox->AddChild(btn_ok);
}
float UserDefinePageDlg::GetPageWidth()
{
  float f;
  FixFloatString(tf_width);
  if (sscanf(tf_width->Text(),"%f",&f) == 0)
  {
    tf_width->SetText("0.0");
    return 0.0f;
  }
  else
  return f;
}
float UserDefinePageDlg::GetPageHeight()
{
  float f;
  FixFloatString(tf_height);
  if (sscanf(tf_height->Text(),"%f",&f) == 0)
  {
    tf_height->SetText("0.0");
    return 0.0f;
  }
  else
  return f;
}
void UserDefinePageDlg::OnOK()
{
  BAlert *myalert;
  char str[256];
  Lock();
  if ((ftarget == NULL) || (fmsg == NULL))
  Quit();
  
  float width  = GetPageWidth();
  float height = GetPageHeight();
  
  if ((width < page_min_width) || (width > page_max_width))
  {
    if (funit == PRINTER_UNIT_INCH)
    sprintf(str,"page width must be between %0.2f and %0.2f inch",page_min_width,page_max_width);
    else
    sprintf(str,"page width must be between %0.2f and %0.2f cm",page_min_width,page_max_width);
    myalert = new BAlert(str,str,"ok");
    myalert->Go();
    Unlock();
    return;
  }
  if ((height < page_min_height) || (height > page_max_height))
  {
    if (funit == PRINTER_UNIT_INCH)
    sprintf(str,"page height must be between %0.2f and %0.2f inch",page_min_height,page_max_height);
    else
    sprintf(str,"page height must be between %0.2f and %0.2f cm",page_min_height,page_max_height);
    myalert = new BAlert(str,str,"ok");
    myalert->Go();
    Unlock();
    return;
  }

  fmsg->AddInt32("result",B_OK);
  fmsg->AddFloat("width",width);
  fmsg->AddFloat("height",height);
  ftarget->PostMessage(fmsg);
  
  Quit();
}
void UserDefinePageDlg::OnCancel()
{
  Lock();
  if ((ftarget == NULL) || (fmsg == NULL))
  Quit();
  fmsg->AddInt32("result",B_ERROR);
  ftarget->PostMessage(fmsg);
  Quit();
  Unlock();
}
void UserDefinePageDlg::OnPageWidthChange()
{
  Lock();
  FixFloatString(tf_width);
  Unlock();
}
void UserDefinePageDlg::FixFloatString(BTextControl *control)
{
  bool founddot = false;
  BString in_str;
  BString out_str = "";;
  in_str.SetTo(control->Text());
  if (in_str[0] == '.')
  out_str << '0';
  int i = 0;
  while (i < in_str.Length())
  {
    if (in_str[i] != '.')
    out_str << in_str[i];
    else
    {
      if (founddot)
      break;
      founddot = true;
      out_str << in_str[i];
    }
    i++;
  }
  control->SetText(out_str.String());
}
void UserDefinePageDlg::OnPageHeightChange()
{
  Lock();
  FixFloatString(tf_height);
  Unlock();
}
void UserDefinePageDlg::SetTo(float width,float height)
{
  char temp[256];
  sprintf(temp,"%0.2f",width);
  tf_width->SetText(temp);
  sprintf(temp,"%0.2f",height);
  tf_height->SetText(temp);
}
